%% Load geometry and show the edges
geometry = importGeometry('geometry.stl');

figure();
pdegplot(geometry,'EdgeLabels','on');
xlabel('X distance (m)');
ylabel('Y distance (m)');

%%

% specify the edge IDs
outside_edges = [2 3 11 13];
dielectric_edges = [7 10 15 14];
electrode1_edges = [4 1 8 6];
electrode2_edges = [9 12 5 16];

% fill in the dielectric 
geometry = addFace(geometry, dielectric_edges);

% show the faces 
pdegplot(geometry,'FaceLabels','on');

%% create the model and look at the mesh
emagmodel = createpde('electromagnetic','electrostatic');
emagmodel.Geometry = geometry;

generateMesh(emagmodel, "HEdge", {[ electrode1_edges electrode2_edges ], 1e-5}); 
figure();
pdeplot(emagmodel);

%% Set material properties
emagmodel.VacuumPermittivity = 8.8541878128E-12;
electromagneticProperties(emagmodel, "Face", 1, "RelativePermittivity", 1);
electromagneticProperties(emagmodel, "Face", 2, "RelativePermittivity", 15);

%% Set conductor 1 to 1V and others to 0, and find the charge on each conductor
electromagneticBC(emagmodel, "Voltage", 1, "Edge", electrode1_edges);
electromagneticBC(emagmodel, "Voltage", 0, "Edge", electrode2_edges);
electromagneticBC(emagmodel, "Voltage", 0, "Edge", outside_edges);

% Solve
R = solve(emagmodel); 

figure()
pdeplot(emagmodel,'XYData',R.ElectricPotential);
xlabel('X distance (m)');
ylabel('Y distance (m)');
c=colorbar(gca());
c.Label.String = 'Electric potential (V)';
axis tight;
axis equal;
hold on;
pdegplot(geometry);

C11 = charge_on_conductor(R, electrode1_edges)
C21 = charge_on_conductor(R, electrode2_edges)

%% Set conductor 2 to 1V and others to 0, and find the charge on each conductor
electromagneticBC(emagmodel, "Voltage", 0, "Edge", electrode1_edges);
electromagneticBC(emagmodel, "Voltage", 1, "Edge", electrode2_edges);
electromagneticBC(emagmodel, "Voltage", 0, "Edge", outside_edges);

% Solve
R = solve(emagmodel); 

figure()
pdeplot(emagmodel,'XYData',R.ElectricPotential);
xlabel('X distance (m)');
ylabel('Y distance (m)');
c=colorbar(gca());
c.Label.String = 'Electric potential (V)';
axis tight;
axis equal;
hold on;
pdegplot(geometry);

C12 = charge_on_conductor(R, electrode1_edges)
C22 = charge_on_conductor(R, electrode2_edges)

%% Form the Maxwell capacitance matrix
C = [C11 C12; 
    C21 C22]

%% Calculate the mutual capacitance
Cm12 = -C12
Cm21 = -C21
Cm11 = C11 - Cm12
Cm22 = C22 - Cm21

% total capacitance measured from conductor 1 to conductor 2 is Cm12 in
% parallel with the self-capacitances Cm11 and Cm22 (in series).
C_total = Cm12 + 1/(1/Cm11 + 1/Cm22)
